/////////////////////////////////////////////////////////////////////////////
/// 
/// \file	AviPlayer.cpp
/// \author	Rob Bateman
///	\brief	This file enables you to load Avi files and use them either
///			as an openGL texture, or to return the pixel data to you
///			in a nice funky form. 
/// 
///			This file uses the windows media library to load the avi files.
///			This kinda ignores the audio of the avi files because I hate 
///			writing sound code outside of DirectAudio. I am thinking that 
///			I may replace this with a better directshow interface (with
///			dolby surround support (hehe)). 
///
///			I've had a few problems with Double Buffering/Draw pixels/
///			NVIDIA and large avi files. The playback seems to dissappear.
///			I think this is a driver issue with the crappy OGL drivers 
///			that I've got. Could be a bug in my code, I don't think so, 
///			but I won't guarentee it. Seems ok for textures and single
///			buffering though.....
///
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
//	Includes
//
/////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <vfw.h>
#include <stdlib.h>
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <vector>
#include <cassert>
#include <iostream>


#pragma comment(lib,"vfw32.lib")
//#pragma comment(lib,"opengl32.lib")
//#pragma comment(lib,"glu32.lib")

using namespace std;

 int avi=0;



/////////////////////////////////////////////////////////////////////////////
//
//	Globals
//
/////////////////////////////////////////////////////////////////////////////

// have the openGL extensions been checked?
bool g_bGL_Checked	  = false;

// is data transfer of BGR pixel data supported?
bool g_bBGR_Supported = false;

/////////////////////////////////////////////////////////////////////////////
///	defines openGL extension GL_EXT_bgra to speed up data transfer of a
///	bitmap to openGL. Quite important cos its slow to manually convert all
///	the colours.

#ifndef GL_EXT_bgra
#define GL_BGR_EXT                        0x80E0
#define GL_BGRA_EXT                       0x80E1
#endif

/////////////////////////////////////////////////////////////////////////////
/// \brief	checks to see if requested gl extension is in driver extensions
/// \param	string	-	the openGL extension string
///	\param	search	-	the gl extension to find
///	\return	true if extension supported
///
bool __IsInString(char *string, const char *search)
{
	unsigned int pos ,i,
		maxpos	= static_cast<unsigned int>( strlen(search) - 1 ),
		len		= static_cast<unsigned int>( strlen(string) );

	for (i=0; i<len; i++)
	{
		if( (i==0) || ((i>1) && string[i-1]=='\n') )
		{
			pos=0;
			while (string[i]!='\n')
			{
				if( string[i]==search[pos] ) pos++;
				if( (pos>maxpos) && string[i+1]=='\n' ) return true;
				i++;
			}
		}
	}
    return false;
}


/////////////////////////////////////////////////////////////////////////////
/// \brief	checks to see if requested gl extension is in driver extensions
/// \param	extString	-	the gl extension to find
///	\return	true if extension supported
///
bool __CheckGLExt( char *extString )
{
	unsigned int length, i;
    char *extensions;

	extensions = (char *) glGetString(GL_EXTENSIONS);
	length = static_cast<unsigned int>( strlen(extensions) ); 
	for(i=0;i<length;i++)
	{
		if(extensions[i] == ' ') extensions[i]='\n';
	}
	if(__IsInString(extensions,extString))
	{
		return true;
	}

	return false;
}




/////////////////////////////////////////////////////////////////////////////
//
//	class CAvi file
//
/////////////////////////////////////////////////////////////////////////////

class CAvi
{
private:

	unsigned int	m_id;					//	a unique id for the avi

	/////////////////////////////////////////////////////
	//	Some variables that hold some useful info and
	//	current places in anim etc.
	/////////////////////////////////////////////////////

	float			m_fCurrentTime;			//	the current time 
	float			m_fTimePerFrame;		//	the time in seconds for each frame
	unsigned int	m_iTexObj;				//	the texture object of the avi file (if being used for an openGL texture)
	unsigned int	m_iTexDimension;		//	the dimension of the texture, ie 128x128 etc
	unsigned int	m_iWidth;				//	the width of the avi video stream
	unsigned int	m_iHeight;				//	the height of the avi video stream
	bool			m_bIsPlaying;			//	is this avifile being used as a texture?
	bool			m_bIsTexture;			//	this is a flag to see if this avi is used as a texture 
	bool			m_bIsBGR;				//	is this avifile being used as a texture?
	char			*m_pTexBuffer;			//	a buffer for the texture data
	char			*m_pAviBuffer;			//	a buffer for the avi data (warning this is liable to be compressed data!)
	unsigned int	m_iCurrFrame;			//	the current fram number
	unsigned int	m_lLastFrame;			//	the last frame number read from the file
	float			m_speed;				//   KGB added speed control				


	/////////////////////////////////////////////////////


	/////////////////////////////////////////////////////
	//	Win32 avi data types and bitmap stuff
	/////////////////////////////////////////////////////
	PAVIFILE		 m_pAviFile;			//	pointer to the avi file header
	PAVISTREAM		 m_pVideoStream;		//	pointer to the video stream
	AVISTREAMINFO    m_pVideoStreamInfo;	//	pointer to a video stream info struct
	LPBITMAPINFO     m_pVideoFormat;		//	the video format of the stream
	BITMAPINFOHEADER m_pCurrentFrame;		//	the image data for the current frame
	PGETFRAME        m_pVideoPGF;			//	a get frame thing....
	HDRAWDIB		 m_hdd;					//	handle to the draw dib object
	HBITMAP			 m_hBitmap;				//	handle to the bitmap
	HDC				 m_hdc;					//	handle to the device context of the bitmap
	/////////////////////////////////////////////////////

public:
	
	void SetSpeed(float speed, unsigned int id);

	/////////////////////////////////////////////////////	
	/// \brief	This function returns to you the id of this Avi file.
	///	\return	the ID of the node.
	///
	unsigned int	 GetID();

	/////////////////////////////////////////////////////
	/// \brief	This function will bind the avi file as the 
	///			the current texture.
	void Bind();

	/////////////////////////////////////////////////////
	/// \brief	is the buffer BGR? or RGB at the moment?
	///
	bool IsBGR();

	/////////////////////////////////////////////////////
	//	Protected functions you need not know about
	/////////////////////////////////////////////////////
	void DetermineVideoFormat();
	void SwitchRGB();
	/////////////////////////////////////////////////////

	void Update(float fTimeInc);
	void Close();
	bool Open(char filename[],unsigned int iSize);

	/////////////////////////////////////////////////////
	/// \brief	ctor
	CAvi();

	/////////////////////////////////////////////////////
	/// \brief	dtor
	~CAvi();

	/////////////////////////////////////////////////////
	/// \brief	this function draws the pixel data at the current 
	///			raster position.
	///
	void Draw();

	/////////////////////////////////////////////////////
	/// \brief	returns the width of the avi video stream
	/// \return	the width
	///
	unsigned int GetWidth ();

	/////////////////////////////////////////////////////
	/// \brief	returns the height of the avi video stream
	/// \return	the height
	///
	unsigned int GetHeight();

	/////////////////////////////////////////////////////
	/// \brief	returns a pointer to the image data
	/// \return	a pointer to the rgb pixel data
	///
	unsigned char* GetPixels();


	/////////////////////////////////////////////////////
	/// \brief	This functions starts playback of the avi file
	/// 
	inline void Play() { m_bIsPlaying=true; }

	/////////////////////////////////////////////////////
	/// \brief	This functions stops playback of the avi file
	/// 
	inline void Stop() { m_bIsPlaying=false; m_fCurrentTime = 0; m_iCurrFrame=0; }
	
	/////////////////////////////////////////////////////
	/// \brief	This functions pauses playback of the avi file
	/// 
	inline void Pause() { m_bIsPlaying=false; }

	/////////////////////////////////////////////////////
	/// \brief	This functions queries the current state of playback
	/// 
	inline bool IsPlaying() const { return m_bIsPlaying; }

	inline void Speed(float speed) {m_speed=speed; }

		
	/////////////////////////////////////////////////////
	/// \brief	This functions reads a chunk of avi data from the file
	/// 
	void ReadChunk();
};


///	a global array of all avi files that are currently open
std::vector< CAvi* > g_aAviFiles;


/////////////////////////////////////////////////////////////////////////////
/// \brief	This functions generates a unique ID for a new avi file
/// 
unsigned int GenIDAvi()
{
	// start with id 0
	unsigned int id=0;

// loop point
jump_label:

	// increment id
	++id;

	// try to find it in the avi file array
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if( *it )
		{
			// if found an avi with this id, try again
			if((*it)->GetID() == id)
				goto jump_label;
		}
	}

	// this id is unused...
	return id;
}

/////////////////////////////////////////////////////////////////////////////
/// \brief	This function returns to you the id of this Avi file.
///	\return	the ID of the node.
///
unsigned int CAvi::GetID()
{
	return m_id;
}




/////////////////////////////////////////////////////////////////////////////
/// \brief	returns the width of the avi video stream
/// \return	the width
///
unsigned int CAvi::GetWidth()
{
	return m_iWidth;
}



/////////////////////////////////////////////////////////////////////////////
/// \brief	returns the height of the avi video stream
/// \return	the height
///
unsigned int CAvi::GetHeight()
{
	return m_iHeight;
}


/////////////////////////////////////////////////////////////////////////////
/// \brief	returns a pointer to the image data
/// \return	a pointer to the rgb pixel data
///
unsigned char* CAvi::GetPixels()
{
	return reinterpret_cast<unsigned char*>(m_pTexBuffer);
}


/////////////////////////////////////////////////////////////////////////////
/// \brief	is the buffer BGR? or RGB at the moment?
///
bool CAvi::IsBGR()
{
	return m_bIsBGR;
}



/////////////////////////////////////////////////////////////////////////////
/// \brief	This function will bind the avi file as the 
///			the current texture.
void CAvi::Bind()
{
	if(m_bIsTexture)
	{
		glBindTexture(GL_TEXTURE_2D,m_iTexObj);
	}
}


/////////////////////////////////////////////////////////////////////////////
/// \brief	This functions starts playback of the avi file
/// 
void CAvi::Draw()
{
	if( g_bBGR_Supported && IsBGR() )
	{
		glDrawPixels( GetWidth(), GetHeight(), GL_BGR_EXT, GL_UNSIGNED_BYTE, GetPixels() );
	}
	else
	{
		if( IsBGR() ) 
		{
			SwitchRGB();
		}
		glDrawPixels( GetWidth(), GetHeight(), GL_RGB, GL_UNSIGNED_BYTE, GetPixels() );
	}
}


/////////////////////////////////////////////////////////////////////////////
//	Construction

CAvi::CAvi()
:	m_pTexBuffer(NULL),
	m_pAviBuffer(NULL),
	m_pAviFile(NULL),
	m_pVideoStream(NULL),
	m_pVideoFormat(NULL),
	m_pVideoPGF(NULL),
	m_fCurrentTime(0),
	m_iCurrFrame(0),
	m_bIsTexture(false),
	m_bIsBGR(true),
	m_iTexObj(0),
	m_speed(1)
{
	m_id = GenIDAvi();
	m_hdc = CreateCompatibleDC(0);
	AVIFileInit();
}


/////////////////////////////////////////////////////////////////////////////
//	Destruction

CAvi::~CAvi()
{
	Close();
	AVIFileExit();
}


/////////////////////////////////////////////////////////////////////////////
//	asm routine to swap Red and Blue bits as fast a possible. Only used
//	if the graphics card does not support GL_BGR_EXT. This is slower than
//	using the extensions, but it the quickest way of doing it

void CAvi::SwitchRGB()
{
	if(m_bIsBGR)
	{
		int iSize;
		m_bIsBGR = false;
		void* b = m_pTexBuffer;
		if(m_bIsTexture)
		{
			iSize = m_iTexDimension*m_iTexDimension ;
		}
		else
		{
			iSize = m_iWidth*m_iHeight ;
		}

		__asm
		{
			mov ecx, iSize
			mov ebx, b
			label:
				mov al,[ebx+0]
				mov ah,[ebx+2]
				mov [ebx+2],al
				mov [ebx+0],ah

				add ebx,3
				dec ecx
				jnz label
		}
	}
}



/////////////////////////////////////////////////////////////////////////////
/// \brief	This function is called whenever a chunk needs to be read from 
///			the avi file.
///
void CAvi::ReadChunk()
{
		//
		//	Pointer to the bitmap data structure of current frame
		//
		LPBITMAPINFOHEADER pBitmapFrame;
		pBitmapFrame = (LPBITMAPINFOHEADER)AVIStreamGetFrame(m_pVideoPGF, m_iCurrFrame);

		//
		//	get a pointer to the actual texture data
		//
		m_pAviBuffer = (char *)pBitmapFrame+pBitmapFrame->biSize+pBitmapFrame->biClrUsed*sizeof(RGBQUAD);

		//
		//	currently in BGR color mode
		//
		m_bIsBGR = true;

		//
		//	if this file is being used as an openGL texture
		//
		if(m_bIsTexture)
		{
			//
			// Convert Data To Requested Bitmap Format
			//
			DrawDibDraw(m_hdd,
				m_hdc,
				0,
				0,
				m_iTexDimension,
				m_iTexDimension,
				pBitmapFrame,
				m_pAviBuffer,
				0,
				0,
				m_iWidth,
				m_iHeight,
				0);
		}
		else
		{

			//
			// Convert Data To Requested Bitmap Format
			//
			DrawDibDraw(m_hdd,
				m_hdc,
				0,
				0,
				m_iWidth,
				m_iHeight,
				pBitmapFrame,
				m_pAviBuffer,
				0,
				0,
				m_iWidth,
				m_iHeight,
				0);
		}



		if(m_bIsTexture)
		{
			//	if BGR data transfers are supported
			//
			if(!g_bBGR_Supported)
			{
				//
				// Update The Texture
				//
				glBindTexture(GL_TEXTURE_2D,m_iTexObj);
				glTexSubImage2D(GL_TEXTURE_2D,
					0,
					0,
					0,
					m_iTexDimension,
					m_iTexDimension,
					GL_BGR_EXT ,
					GL_UNSIGNED_BYTE,
					m_pTexBuffer);
			}
			else
			{
				//
				//	Flip Red & Blue pixels (bmp stored bgr)
				//
				SwitchRGB();

				//
				// Update The Texture
				//
				glBindTexture(GL_TEXTURE_2D,m_iTexObj);
				glTexSubImage2D(GL_TEXTURE_2D,
					0,
					0,
					0,
					m_iTexDimension,
					m_iTexDimension,
					GL_RGB,
					GL_UNSIGNED_BYTE,
					m_pTexBuffer);
			}
		}
}


/////////////////////////////////////////////////////////////////////////////
///	\brief	This function updates all of the loaded sound files (only if 
///			currently playing). The time increment is needed to determine the 
///			current time of the avi files.
/// \param	The time (in seconds) since last updated.
///
void CAvi::Update(float fTimeInc)
{
	if(m_bIsPlaying)
	{

		//
		//	increment the current time
		//
		m_fCurrentTime+=fTimeInc;
		
		//
		//	if gone past end of avi file
		//
		if(m_fCurrentTime>m_fTimePerFrame)
		{
			//
			//	while avi time greater than file length,
			//
			while(m_fCurrentTime>m_fTimePerFrame) 
			{
				//
				//	recalculate durrent time
				//
				ReadChunk();
				m_fCurrentTime-=m_fTimePerFrame;
			}
			//
			//	move to next frame
			//
			m_iCurrFrame=m_iCurrFrame+m_speed;

			if(m_iCurrFrame >= m_lLastFrame)
			{
				m_iCurrFrame = 0;
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
/// \brief	Close the stream, release everything we've accumulated along the way.
///
void CAvi::Close()
{
	//
	//	Release all avi stuff
	//
	if(m_pAviFile)
		AVIFileRelease  (m_pAviFile);
	if(m_pVideoStream)
		AVIStreamRelease(m_pVideoStream);

	//
	//	delete all useless data
	//
	delete [] ((LPBYTE)m_pVideoFormat);

	DeleteObject(m_hBitmap);
	DrawDibClose(m_hdd);

	m_pAviFile     = NULL;
	m_pVideoStream = NULL;
	m_pVideoFormat = NULL;
	m_iTexDimension = 0;
	m_iWidth		= 0;
	m_iHeight		= 0;
	m_fCurrentTime	= 0;
	m_fTimePerFrame = 0;
	m_iCurrFrame	= 0;
	glDeleteTextures(1,&m_iTexObj);
}


/////////////////////////////////////////////////////////////////////////////
///	\brief	An Avi file can contain a varying number of audio and video streams.
///			Essentually this function qeuries the avi file to see what video
///			streams are in the file (if any).

void CAvi::DetermineVideoFormat()
{
	LONG lSize = 40;

	//
	//	Allocate space for required chunk and read in the data
	//	- always 40bytes I reckon - I may be wrong
	//
	LPBYTE pData = new BYTE[lSize];
	if(!pData)
	{
		printf("{ CAvi::DetermineVideoFormat } Failed to allocate memory\n");
		exit(0);
	}

	AVIStreamReadFormat(m_pVideoStream, AVIStreamStart(m_pVideoStream), pData, &lSize);

	//
	//	Set the format pointer to point to the correct chunk.
	//
	m_pVideoFormat = (LPBITMAPINFO)pData;
}



/////////////////////////////////////////////////////////////////////////////
///	Opens and initialises the avi stream, the bitmap surface to draw to, and
///	the openGL texture object. Takes the name of the avi, and the size of
///	texture you want. (ie, 64 would give you a 64x64 texture). Passing in 
///	zero will just load the avi rather than making a texture
///
bool CAvi::Open(char filename[],unsigned int iSize)
{
	m_hdd      = DrawDibOpen();

	//
	//	Open up the *.avi file
	//
	switch(AVIFileOpen( &m_pAviFile , filename , OF_READ , NULL ))
	{
	case AVIERR_BADFORMAT:
		printf("File Corrupt or Not In Existance! %s\n",filename);
		return false;

	case AVIERR_MEMORY:
		printf("File out of memory %s\n",filename);
		return false;

	case AVIERR_FILEREAD:
		printf("Error reading avifile %s\n",filename);
		return false;

	case AVIERR_FILEOPEN:
		printf("Error avifile already open %s\n",filename);
		return false;

	case REGDB_E_CLASSNOTREG:
		printf("Error: Avi type unsupported %s\n",filename);
		return false;

	default:
		break;
	}

	//
	//	get hold of the audio and video streams
	//
	AVIFileGetStream(m_pAviFile,
					&m_pVideoStream,
					 streamtypeVIDEO,
					 0);

	//
	//	get detailed info on both streams. - dont know what to do with audio yet
	//
	if(m_pVideoStream)
	{
		DetermineVideoFormat();
		AVIStreamInfo(m_pVideoStream,&m_pVideoStreamInfo,sizeof(m_pVideoStreamInfo));
		m_iWidth	= m_pVideoStreamInfo.rcFrame.right  - m_pVideoStreamInfo.rcFrame.left;
		m_iHeight	= m_pVideoStreamInfo.rcFrame.bottom - m_pVideoStreamInfo.rcFrame.top ;
	}

	if(iSize != 0)
	{
		//
		//	Set up a context to draw to...
		//	this will take a rescaled version of whats in the avi
		//
		m_pCurrentFrame.biHeight = iSize;
		m_pCurrentFrame.biWidth  = iSize;
	}
	else
	{
		//
		//	Set up a context to draw to...
		//	this will take a rescaled version of whats in the avi
		//
		m_pCurrentFrame.biHeight = m_iHeight;
		m_pCurrentFrame.biWidth  = m_iWidth;
	}
	m_pCurrentFrame.biSize   = sizeof (BITMAPINFOHEADER);
	m_pCurrentFrame.biPlanes = 1;
	m_pCurrentFrame.biBitCount = 24;
	m_pCurrentFrame.biCompression = BI_RGB;


	//
	//	Create the rescaled dib
	//
	m_hBitmap = CreateDIBSection(m_hdc,
		(BITMAPINFO*)(&m_pCurrentFrame),
		DIB_RGB_COLORS,
		(void**)(&m_pTexBuffer),
		NULL,
		NULL);

	SelectObject(m_hdc, m_hBitmap);

	if(iSize != 0)
	{
		//
		//	Sort out the openGL texture object
		//
		glEnable(GL_TEXTURE_2D);
		glGenTextures(1,&m_iTexObj);
		glBindTexture  (GL_TEXTURE_2D, m_iTexObj);
		glPixelStorei  (GL_UNPACK_ALIGNMENT, 1);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

		//
		//	Create texture object
		//
		glTexImage2D(	GL_TEXTURE_2D,
						0,
						GL_RGB,
						iSize,
						iSize,
						0,
						GL_RGB,
						GL_UNSIGNED_BYTE,
						m_pTexBuffer );

		m_bIsTexture = true;

	}
	//
	//	Small check to see if the file isn't corrupt
	//
	m_pVideoPGF = AVIStreamGetFrameOpen(m_pVideoStream, NULL);
	if (m_pVideoPGF==NULL)
	{
		printf( "Couldn't retrieve first AVI frame\n");
		return false;
	}

	//
	//	Work out some useful data
	//
	m_lLastFrame = AVIStreamLength(m_pVideoStream);
	m_fTimePerFrame = (float)(AVIStreamSampleToTime(m_pVideoStream,m_lLastFrame)/m_lLastFrame)/1000.0f;
	m_iTexDimension = iSize;
	m_iCurrFrame = 0;
	
	ReadChunk();

	return true;

}





//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
//	Avi Initialisation / Uninitialisation
//
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function does some initialisation. It's not vital that 
///			this function be called, however it is recommended. Basically
///			the only thing it does is check to see if the graphics card 
///			supports BGR pixel data transfers, in which case the data need
///			not be flipped to RGB, saving a bit of processing time.
///
void AviInit(void)
{
	// only check the opengl extensions once
	if(!g_bGL_Checked)
	{
		 //if bgra pixel data is supported
		g_bBGR_Supported = __CheckGLExt("GL_EXT_bgra");

		 //checked the extension
		g_bGL_Checked = true;
	}
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function cleans up any data still floating about when you
///			are wanting to exit your application.
///
void AviCleanup(void)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			(*it)->Close();
			delete *it;
		}
	}

	g_aAviFiles.resize(0);
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function opens the specified avi file for use as a texture.
///			You must also specify the dimension of the texture to scale the
///			avi to, ie, specify 128 if you wish the avi to be scaled to 
///			dimensions of 128x128. This value must be a power of two.
/// 
///			If you wish to bind this avi file as a texture, then call the 
///			function AviTextureBind()
///
///	\param	filename	-	the name of the file to load
///	\param	dimension	-	the size of the texture to use, ie 128 for 128x128
/// \return	a unique id that you will use to refer to the avi
///
unsigned int AviTextureOpen(char filename[],unsigned int dimension)
{
	unsigned int dim = 128;

	// make sure dimension is valid for texture data
	if( dimension == 2    ) dim = 2;
	if( dimension == 4    ) dim = 4;
	if( dimension == 8    ) dim = 8;
	if( dimension == 16   ) dim = 16;
	if( dimension == 32   ) dim = 32;
	if( dimension == 64   ) dim = 64;
	if( dimension == 128  ) dim = 128;
	if( dimension == 256  ) dim = 256;
	if( dimension == 512  ) dim = 512;
	if( dimension == 1024 ) dim = 1024;
	if( dimension == 2048 ) dim = 2048;
	if( dimension == 4096 ) dim = 4096;



	// create new avi structure
	CAvi *avi = new CAvi;

	// check allocation
	assert(avi);

	// open the file
	if(avi->Open(filename,dim))
	{
		// add the file to the back of the list
		g_aAviFiles.push_back(avi);
	}
	else
	{
		delete avi;
		return 0;
	}
	return avi->GetID();
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
void AviTextureBind(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				
				(*it)->Bind();
				return;
			}
		}
	}
}



//////////////////////////////////////////////////////////////////////////
/// \brief	This function opens the specified avi file and allows you to
///			query the avi frame data directly. Possible uses for this may 
///			be passing the data to glDrawPixels for game intro screnes etc.
///
///			You can query the size of the avi file using the functions AviGetWidth
///			and AviGetHeight. 
/// 
///			Getting acces to the pixel data is then simply a case of calling
///			AviGetPixels(). The data will always be returned as an array of
///			RGB pixel data (the size of which is AviGetHeight() * AviGetWidth() * 3 )
///
///	\param	filename	-	the name of the file to load
/// \return	a unique id that you will use to refer to the avi
///
unsigned int	AviOpen(char filename[])
{

	// create new avi structure
	CAvi *avi = new CAvi;

	// check allocation
	assert(avi);

	// open the file
	if(avi->Open(filename,0))
	{
		// add the file to the back of the list
		g_aAviFiles.push_back(avi);
	}
	else
	{
		delete avi;
		return 0;
	}
	return avi->GetID();
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function uses glDrawPixels to draw the image data held by
///			the specified Avi File. 
/// \param	id	-	the id of the avi to draw
///	\param	x	-	the x-coord location of the display
///	\param	y	-	the y-coord location of the display
///
void	AviDraw(unsigned int id,int x,int y)
{
	glDrawBuffer(GL_BACK);
	glRasterPos2i(x,y);

	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();
	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				
				(*it)->Draw();
				return;
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the width of an avi file loaded
///			specifically for getting pixel data. This allows you to compute the 
///			image size and pixel array size.
/// \param	id	-	the id of the avi to qeury
/// \return	the width of the avi file
///
unsigned int AviGetWidth(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				
				return (*it)->GetWidth();
			}
		}
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the height of an avi file loaded
///			specifically for getting pixel data. This allows you to compute the 
///			image size and pixel array size.
/// \param	id	-	the id of the avi to qeury
/// \return	the width of the avi file
///
unsigned int	AviGetHeight(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				return (*it)->GetHeight();
			}
		}
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the pixel data array of the 
///			current frame of the avi.
/// \param	id	-	the id of the avi to qeury
/// \return	a pointer to the pixel data or NULL
///
unsigned char* AviGetPixels(unsigned int id)
{

	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				return (*it)->GetPixels();
			}
		}
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////////
//
//	Avi Close
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
void AviClose(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				(*it)->Close();
				delete *it;
				g_aAviFiles.erase(it);
				return;
			}
		}
	}
}
//////////////////////////////////////////////////////////////////////////
//
//	Avi Update
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function advances any avi files that are currently playing
///	\param	fTimeInc	-	the time increment which to apply to the avi files
///	\note	This should probably be called as part of the idle proceesing of your
///			application
///
void AviUpdate(float fTimeInc)
{

	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();
	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			
			(*it)->Update(fTimeInc);
				
		}
	}
}
//////////////////////////////////////////////////////////////////////////
//
//	Avi Playback Controls
//
//////////////////////////////////////////////////////////////////////////
void SetSpeed(unsigned int id, float speed)
{
std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
				
			{
				
				(*it)->Speed(speed);
				return;
			}
		}
	}	



}


void AviPlay(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
				
			{
				
				(*it)->Play();
				return;
			}
		}
	}
}

void AviPause(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				(*it)->Pause();
				return;
			}
		}
	}
}

void AviStop(unsigned int id)
{
	std::vector< CAvi* >::iterator it = g_aAviFiles.begin();

	for( ; it != g_aAviFiles.end(); ++it )
	{
		if(*it)
		{
			if((*it)->GetID() == id)
			{
				(*it)->Stop();
				return;
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//KGB added functions and stuff//

// id of the avi file
 int aviID[10];

 AviLoadFiles()
{
		// load the avi file
aviID[0]= AviTextureOpen("a day mi skate fire remix MidiVid.avi",128);

aviID[1]=AviTextureOpen("shaolinMidiVid.avi",128);

aviID[2]=AviTextureOpen("FreeRunningEditMidiVid.avi",128);
return 1;

}





















































































