//////////////////////////////////////////////////////////////////////////
/// 
/// \file	AviPlayer.h
/// \author	Rob Bateman
///	\brief	This file enables you to load Avi files and use them either
///			as an openGL texture, or to return the pixel data to you
///			in a nice funky form. 
/// 
///			This file uses the windows media library to load the avi files.
///			This kinda ignores the audio of the avi files because I hate 
///			writing sound code outside of DirectAudio. I am thinking that 
///			I may replace this with a better directshow interface (with
///			dolby surround support (hehe)).
///
///			I've had a few problems with Double Buffering/Draw pixels/
///			NVIDIA and large avi files. The playback seems to dissappear.
///			I think this is a driver issue with the crappy OGL drivers 
///			that I've got. Could be a bug in my code, I don't think so, 
///			but I won't guarentee it. Seems ok for textures and single
///			buffering though.....
///
//////////////////////////////////////////////////////////////////////////

#pragma once

#ifndef _AVI_LOADER_H_
#define _AVI_LOADER_H_



//////////////////////////////////////////////////////////////////////////
//
//	Avi Initialisation / Uninitialisation
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function does some initialisation. It's not vital that 
///			this function be called, however it is recommended. Basically
///			the only thing it does is check to see if the graphics card 
///			supports BGR pixel data transfers, in which case the data need
///			not be flipped to RGB, saving a bit of processing time.
///
extern void AviInit(void);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function cleans up any data still floating about when you
///			are wanting to exit your application.
///
extern void AviCleanup(void);





//////////////////////////////////////////////////////////////////////////
//
//	Avi OpenGL Texture Utilities
///	---------------------------------------------
///
///	// example, displaying an avi file
/// #include <windows.h>
///	#include <GL/glut.h>
///	#include "AviPlayer.h"
/// #include "FrameTimer.h"
///
///	// id of the avi file
/// unsigned int aviID = 0;
///
///
/// // custom openGL initialisation
/// void init() {
///
///		// we require a high resolution frame timer for this
///		InitFrameTimer();
///		
///		// init the Avi Lib
///		AviInit();
///
///		// load the avi file
///		aviID = AviTextureOpen("SomeFile.avi",256);
///
///		// start playback
///		AviPlay( aviID );
/// 
///		// enable texturing
///		glEnable(GL_TEXTURE_2D);
///	}
///
///	// on display
///	void display() {
///
///		// clear the buffer
///		glClear(GL_COLOR_BUFFER_BIT);
///
///		// clear any transforms
///		glLoadIdentity();
///
///		// set camera position
///		gluLookAt(0,0,5,0,0,0,0,1,0);
///
///		// bind the avi file as an openGL texture
///		AviTextureBind( aviID );
///
///		// draw a polygon on which the texture will be displayed
///		glBegin(GL_QUADS);
///	
///			glColor3f(1.0f,1.0f,1.0f);
///			glTexCoord2f(1.0f,1.0f);
///			glVertex3f(2.0f,2.0f,0.0f);
///
///			glTexCoord2f(0.0f,1.0f);
///			glVertex3f(-2.0f,2.0f,0.0f);
///
///			glTexCoord2f(0.0f,0.0f);
///			glVertex3f(-2.0f,-2.0f,0.0f);
///
///			glTexCoord2f(1.0f,0.0f);
///			glVertex3f(2.0f,-2.0f,0.0f);
///
///		glEnd();
///
///		// swap the buffers
///		glutSwapBuffers();
///	}
///
///	// on resize  -  puts the display into 2D orthographic mode
///	void resize(int w, int h) 
///	{
///		glViewport(0,0,w,h);
///		glMatrixMode(GL_PROJECTION);
///		glLoadIdentity();
///		gluPerspective(45,(float)w/((h!=0)?h:1.0f),1,100);
///		glMatrixMode(GL_MODELVIEW);
///	}
///
///	// on idle
///	void idle() 
///	{
///		// update the frametimer
///		UpdateFrameTime();
///
///		// update all of the loaded avi files
///		AviUpdate( GetFrameTime() );
///		
///		// redraw the screen
///		glutPostRedisplay();
///	}
///
///	int main( int argc, char **argv )
///	{
///		glutInit(&argc,argv);
///		glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE);
///		glutInitWindowPosition(100,100);
///		glutInitWindowSize(800,600);
///		glutCreateWindow("Avi Example");
///		init();
///		glutDisplayFunc(display);
///		glutReshapeFunc(resize);
///		glutIdleFunc(idle);
///		glutMainLoop();
///
///		// clean up all avi data
///		AviCleanup();
///	}
//
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
/// \brief	This function opens the specified avi file for use as a texture.
///			You must also specify the dimension of the texture to scale the
///			avi to, ie, specify 128 if you wish the avi to be scaled to 
///			dimensions of 128x128. This value must be a power of two.
/// 
///			If you wish to bind this avi file as a texture, then call the 
///			function AviTextureBind()
///
///	\param	filename	-	the name of the file to load
///	\param	dimension	-	the size of the texture to use, ie 128 for 128x128
/// \return	a unique id that you will use to refer to the avi
///
extern unsigned int	AviTextureOpen(char filename[],unsigned int dimension);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
extern void	AviTextureBind(unsigned int id);





//////////////////////////////////////////////////////////////////////////
///
///	Avi Functions when the pixel data is required 
///	---------------------------------------------
///	
///	// example, displaying an avi file
///	#include <windows.h>
///	#include <GL/glut.h>
///	#include "AviPlayer.h"
///	#include "FrameTimer.h"
///	
///	// id of the avi file
///	unsigned int aviID = 0;
///	
///	
///	// custom openGL initialisation
///	void init() {
///	
///		// we require a high resolution frame timer for this
///		InitFrameTime();
///	
///		// init the Avi Lib
///		AviInit();
///	
///		// load the avi file
///		aviID = AviOpen("clock.avi");
///	
///		// make sure file was loaded
///		if(aviID == 0)
///			exit(0);
///	
///		// start playback
///		AviPlay( aviID );
///	}
///	
///	// on display
///	void display() {
///	
///		glDrawBuffer(GL_BACK);
///	
///		// clear the screen 
///		glClear(GL_COLOR_BUFFER_BIT);
///		
///		// draw the avi file (this will be quicker than calling the GetPixels method)
///		AviDraw( aviID, 0, AviGetHeight(aviID) );
///	
///		// swap the buffers
///		glutSwapBuffers();
///	}
///	
///	// on resize  -  puts the display into 2D orthographic mode
///	void resize(int w, int h) {
///	
///		// set the viewport
///		glViewport(0,0,w,h);
///	
///		// about to change the projection matrix...
///		glMatrixMode(GL_PROJECTION);
/// 
///		// load the identity matrix
///		glLoadIdentity();
/// 
///		// set the orthographic projection
///		glOrtho( 0, w,h,0, 1, -1);
///	}
///	
///	// on idle
///	void idle() {
///	
///		// update the frametimer
///		UpdateFrameTime();
///	
///		// update all of the loaded avi files
///		AviUpdate( GetFrameTime() );
///	
///		// redraw the screen
///		glutPostRedisplay();
///	}
///	
///	int main( int argc, char **argv )
///	{
///		glutInit(&argc,argv);
///		glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE);
///		glutInitWindowPosition(300,100);
///		glutInitWindowSize(322,322);
///		glutCreateWindow("Avi Example");
///		init();
///		glutDisplayFunc(display);
///		glutReshapeFunc(resize);
///		glutIdleFunc(idle);
///		glutMainLoop();
///	
///		// clean up all avi data
///		AviCleanup();
///	}
///
//////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
/// \brief	This function opens the specified avi file and allows you to
///			query the avi frame data directly. Possible uses for this may 
///			be passing the data to glDrawPixels for game intro screnes etc.
///
///			You can query the size of the avi file using the functions AviGetWidth
///			and AviGetHeight. 
/// 
///			Getting acces to the pixel data is then simply a case of calling
///			AviGetPixels(). The data will always be returned as an array of
///			RGB pixel data (the size of which is AviGetHeight() * AviGetWidth() * 3 )
///
///	\param	filename	-	the name of the file to load
/// \return	a unique id that you will use to refer to the avi
///
extern unsigned int	AviOpen(char filename[]);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function uses glDrawPixels to draw the image data held by
///			the specified Avi File. 
/// \param	id	-	the id of the avi to draw
///	\param	x	-	the x-coord location of the display
///	\param	y	-	the y-coord location of the display
///
extern void	AviDraw(unsigned int id,int x,int y);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the width of an avi file loaded
///			specifically for getting pixel data. This allows you to compute the 
///			image size and pixel array size.
/// \param	id	-	the id of the avi to qeury
/// \return	the width of the avi file
///
extern unsigned int AviGetWidth(unsigned int id);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the height of an avi file loaded
///			specifically for getting pixel data. This allows you to compute the 
///			image size and pixel array size.
/// \param	id	-	the id of the avi to qeury
/// \return	the width of the avi file
///
extern unsigned int	AviGetHeight(unsigned int id);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function can be used to retrieve the pixel data array of the 
///			current frame of the avi.
/// \param	id	-	the id of the avi to qeury
/// \return	a pointer to the pixel data or NULL
///
extern unsigned char* AviGetPixels(unsigned int id);




//////////////////////////////////////////////////////////////////////////
//
//	Avi Close
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
extern void	AviClose(unsigned int id);




//////////////////////////////////////////////////////////////////////////
//
//	Avi Update
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function advances any avi files that are currently playing
///	\param	fTimeInc	-	the time increment which to apply to the avi files
///	\note	This should probably be called as part of the idle proceesing of your
///			application
///
extern void	AviUpdate(float fTimeInc);




//////////////////////////////////////////////////////////////////////////
//
//	Avi Playback Controls
//
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
extern void	AviPlay(unsigned int id);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
extern void	AviPause(unsigned int id);

//////////////////////////////////////////////////////////////////////////
/// \brief	This function closes the specified file
/// \param	id	-	the id of the avi to close
///
extern void AviStop(unsigned int id);


extern void SetSpeed(unsigned int id, float speed);

int AviLoadFiles();


#endif
