#ifndef __POINT3_H__
#define __POINT3_H__

#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif

#include <GL/gl.h>
#include <iostream>

#include "Vector.h"
namespace GraphicsLib {
/*! class to encapsulate x,y,z points */

class Point3
{

/*! << extraction operator to prints out the point in the format [x,y,z]
   \param std::ostream & output the outstream class pointer
   \param const Point3 &s the point to print out 
*/
friend std::ostream& operator<<(std::ostream& output, const GraphicsLib::Point3& s);
/*! >> insertion operator to read in the point data x -> y -> z
   \param std::istream & input the instream class pointer
   \param const Point3 s the point to read in to
*/
friend std::istream& operator>>(std::istream& input, Point3 &s);
public:
/*! x value of point */
 GLfloat x;
/*! y value of point */
 GLfloat y;
/*! z value of point */
 GLfloat z;
/*! set the point values
 * \param float dx the new x value
 * \param float dy the new y value
 * \param float dz the new z value */
 inline void set(GLfloat dx, GLfloat dy, GLfloat dz){x = dx; y = dy; z = dz;}
/*! set the point values from another point
 * \param Point3 p new point */
 inline void set(Point3 p){x = p.x; y = p.y; z = p.z;}
/*! constructor using paramerer values
 * \param float xx x value
 * \param float yy y value
 * \param float zz z value */
 inline Point3(GLfloat xx,GLfloat yy, GLfloat zz){x = xx; y = yy; z = zz;}
/*default constructor set all values to 0 */
 inline Point3(){x = y = z = 0;}
/*!build a 4 tuple with w=1 for the 4th element
 * \param float v[] the new vector */
 inline void build4tuple(GLfloat v[])
  {// load 4-tuple  v[3] = 1 for homogeneous
   v[0] = x; v[1] = y; v[2] = z; v[3] = 1.0f;
  }
/*! math operator for adding two points
\param const Point3 &rhs the right hand side argument
\returns Point3 the result of this+rhs */
Point3 operator+(const Point3& rhs);
/*! math operator for adding point and a vector
\param const Vector &rhs the right hand side argument
\returns Point3 the result of this+rhs */
Point3 operator+(const Vector& rhs);
/*! math operator for adding a point to the current point
\param const Point3 &rhs the right hand side argument
*/
void operator+=(const Point3 &rhs);
/*! math operator for adding a vector to the current point
\param const Point3 &rhs the right hand side argument
*/
void operator+=(const Vector &rhs);
/*! math operator for subtracting two points
\param const Point3 &rhs the right hand side argument
\returns Vector the result of this-rhs */
Vector operator-(const Point3& rhs);
/*! math operator for subtracting a Vector from a point
\param const Vector &rhs the right hand side argument
\returns Point3 the result of this-rhs */
Point3 operator-(const Vector& rhs);
/*! math operator for subtracting a point from the current point
\param const Point3 &rhs the right hand side argument
*/
void operator-=(const Point3 &rhs);
/*! math operator for subtracting a Vector from the current point
\param const Vector &rhs the right hand side argument
*/
void operator-=(const Vector &rhs);
/*! math operator for multiplying a point by a float scalar
\param GLfloat &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(GLfloat rhs);
/*! math operator for multiplying the current point by a float scalar
\param GLfloat &rhs the right hand side argument
*/
void operator*=(GLfloat rhs);
/*! math operator for multiplying a point by matrix
\param Matrix &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(const Matrix &rhs);
/*! math operator for multiplying a point by a Vector
\param Vector &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/
Point3 operator*(const Vector &rhs);
/*! math operator for multiplying a point by a Point
\param Point3 &rhs the right hand side argument
\returns Point3 the result of this * rhs;
*/

Point3 operator*(const Point3 &rhs);
/*! math operator for dividing a Point by a float scalar
\param GLfloat &rhs the right hand side argument
\returns Point3 the result of this / rhs;
*/
Point3 operator/(GLfloat rhs);
/*! relation operator == compare two points
\param const Point3 &rhs
\returns true if x==rhs.x && y==rhs.y && z==rhs.z else false
*/
bool operator==(const Point3& rhs);
/*! relation operator != compare two points
\param const Point3 &rhs
\returns true if x!=rhs.x && y!=rhs.y && z!=rhs.z else false
*/
bool operator!=(const Point3& rhs);
/*! relation operator >= compare two points
\param const Point3 &rhs
\returns true if x>=rhs.x && y>=rhs.y && z>=rhs.z else false
*/
bool operator>=(const Point3& rhs);
/*! relation operator > compare two points
\param const Point3 &rhs
\returns true if x>rhs.x && y>rhs.y && z>rhs.z else false
*/
bool operator>(const Point3& rhs);
/*! relation operator <= compare two points
\param const Point3 &rhs
\returns true if x<=rhs.x && y<=rhs.y && z<=rhs.z else false
*/
bool operator<=(const Point3& rhs);
/*! relation operator < compare two points
\param const Point3 &rhs
\returns true if x<rhs.x && y<rhs.y && z<rhs.z else false
*/
bool operator<(const Point3& rhs);
/*! use the point as a glVertex */
void Vertex(void);
/*! use glTranslate to move to the current point position */
void Translate(void);
/*! use the current point as a normal */
void Normal(void);
/*! Normalize the current point */
void normalize(void);
};

}
  
#endif
